@@ -123,6 +123,8 @@ struct cmd_opt
     int skip_write_error;
     int write_direct_io;
     int read_direct_io;
+    int binary_prefix;
+    int prog_second;
     unsigned int buffer_size;
     off_t offset;
     unsigned long fresh;
